@@ -22,12 +22,15 @@ import io
 from string import Formatter
 from datetime import datetime
 from difflib import unified_diff
+from tempfile import NamedTemporaryFile
 
 import sys
 import codecs
-sys.stdout = codecs.getwriter('utf8')(sys.stdout)
 
-__VERSION__ = '0.4.0'
+if sys.version_info[0] == 2:
+    sys.stdout = codecs.getwriter('utf-8')(sys.stdout)
+
+__VERSION__ = '0.4.1'
 
 DESCRIPTION = 'bumpversion: v{} (using Python v{})'.format(
     __VERSION__,
@@ -35,7 +38,8 @@ DESCRIPTION = 'bumpversion: v{} (using Python v{})'.format(
 )
 
 import logging
-logger = logging.getLogger("bumpversion")
+logger = logging.getLogger("bumpversion.logger")
+logger_list = logging.getLogger("bumpversion.list")
 
 from argparse import _AppendAction
 class DiscardDefaultIfSpecifiedAppendAction(_AppendAction):
@@ -54,6 +58,16 @@ class DiscardDefaultIfSpecifiedAppendAction(_AppendAction):
 
 class BaseVCS(object):
 
+    @classmethod
+    def commit(cls, message):
+        f = NamedTemporaryFile('wb', delete=False)
+        f.write(message.encode('utf-8'))
+        f.close()
+        subprocess.check_output(cls._COMMIT_COMMAND + [f.name], env=dict(
+            list(os.environ.items()) + [('HGENCODING', 'utf-8')]
+        ))
+        os.unlink(f.name)
+
     @classmethod
     def is_usable(cls):
         try:
@@ -72,6 +86,7 @@ class BaseVCS(object):
 class Git(BaseVCS):
 
     _TEST_USABLE_COMMAND = ["git", "rev-parse", "--git-dir"]
+    _COMMIT_COMMAND = ["git", "commit", "-F"]
 
     @classmethod
     def assert_nondirty(cls):
@@ -126,10 +141,6 @@ class Git(BaseVCS):
     def add_path(cls, path):
         subprocess.check_output(["git", "add", "--update", path])
 
-    @classmethod
-    def commit(cls, message):
-        subprocess.check_output(["git", "commit", "-m", message.encode('utf-8')])
-
     @classmethod
     def tag(cls, name):
         subprocess.check_output(["git", "tag", name])
@@ -138,6 +149,7 @@ class Git(BaseVCS):
 class Mercurial(BaseVCS):
 
     _TEST_USABLE_COMMAND = ["hg", "root"]
+    _COMMIT_COMMAND = ["hg", "commit", "--logfile"]
 
     @classmethod
     def latest_tag_info(cls):
@@ -160,10 +172,6 @@ class Mercurial(BaseVCS):
     def add_path(cls, path):
         pass
 
-    @classmethod
-    def commit(cls, message):
-        subprocess.check_output(["hg", "commit", "-m", message.encode('utf-8')])
-
     @classmethod
     def tag(cls, name):
         subprocess.check_output(["hg", "tag", name])
@@ -416,14 +424,27 @@ def main(original_args=None):
         '--verbose', action='count', default=0,
         help='Print verbose logging to stderr', required=False)
 
+    parser1.add_argument(
+        '--list', action='store_true', default=False,
+        help='List machine readable information', required=False)
+
     known_args, remaining_argv = parser1.parse_known_args(args)
 
+    logformatter = logging.Formatter('%(message)s')
+
     if len(logger.handlers) == 0:
         ch = logging.StreamHandler(sys.stderr)
-        logformatter = logging.Formatter('%(message)s')
         ch.setFormatter(logformatter)
         logger.addHandler(ch)
 
+    if len(logger_list.handlers) == 0:
+       ch2 = logging.StreamHandler(sys.stdout)
+       ch2.setFormatter(logformatter)
+       logger_list.addHandler(ch2)
+
+    if known_args.list:
+          logger_list.setLevel(1)
+
     log_level = {
         0: logging.WARNING,
         1: logging.INFO,
@@ -444,15 +465,19 @@ def main(original_args=None):
     if 'current_version' in vcs_info:
         defaults['current_version'] = vcs_info['current_version']
 
-    config = None
-    if os.path.exists(known_args.config_file):
-        config = RawConfigParser()
+    config = RawConfigParser('')
+    config.add_section('bumpversion')
+
+    config_file_exists = os.path.exists(known_args.config_file)
+
+    if config_file_exists:
+
         config.readfp(io.open(known_args.config_file, 'rt', encoding='utf-8'))
 
         log_config = StringIO()
         config.write(log_config)
-        logger.info("Reading config file {}:".format(known_args.config_file))
 
+        logger.info("Reading config file {}:".format(known_args.config_file))
         logger.info(log_config.getvalue())
 
         defaults.update(dict(config.items("bumpversion")))
@@ -591,6 +616,8 @@ def main(original_args=None):
         if vcs.is_usable():
             vcs.assert_nondirty()
             break
+        else:
+            vcs = None
 
     # make sure files exist and contain version string
 
@@ -620,7 +647,13 @@ def main(original_args=None):
             "Would change" if args.dry_run else "Changing",
             path,
         ))
-        logger.info("\n".join(list(unified_diff(before.splitlines(), after.splitlines(), lineterm="", fromfile="a/"+path, tofile="b/"+path))))
+        logger.info("\n".join(list(unified_diff(
+            before.splitlines(),
+            after.splitlines(),
+            lineterm="",
+            fromfile="a/"+path,
+            tofile="b/"+path
+        ))))
 
         if not args.dry_run:
             with io.open(path, 'wt', encoding='utf-8') as f:
@@ -628,77 +661,89 @@ def main(original_args=None):
 
     commit_files = files
 
-    if config:
-        config.remove_option('bumpversion', 'new_version')
+    config.set('bumpversion', 'new_version', args.new_version)
 
-        config.set('bumpversion', 'current_version', args.new_version)
+    for key, value in config.items('bumpversion'):
+        logger_list.info("{}={}".format(key, value))
 
-        s = StringIO()
+    config.remove_option('bumpversion', 'new_version')
 
-        try:
-            config.write(s)
+    config.set('bumpversion', 'current_version', args.new_version)
 
-            logger.info("{} to config file {}:".format(
-                "Would write" if args.dry_run else "Writing",
-                known_args.config_file,
-            ))
-            logger.info(log_config.getvalue())
+    new_config = StringIO()
 
-            if not args.dry_run:
-                with io.open(known_args.config_file, 'wb') as f:
-                    f.write(s.getvalue().encode('utf-8'))
+    try:
+        write_to_config_file = (not args.dry_run) and config_file_exists
 
-        except UnicodeEncodeError:
-            warnings.warn(
-                "Unable to write UTF-8 to config file, because of an old configparser version. "
-                "Update with `pip install --upgrade configparser`."
-            )
-
-        commit_files.append(known_args.config_file)
-
-    if args.commit:
-
-        assert vcs.is_usable(), "Did find '{}' unusable, unable to commit.".format(vcs.__name__)
-
-        logger.info("Preparing {} commit".format(vcs.__name__))
-
-        for path in commit_files:
-
-            logger.info("{} changes in file '{}' to {}".format(
-                "Would add" if args.dry_run else "Adding",
-                path,
-                vcs.__name__,
-            ))
-
-            if not args.dry_run:
-                vcs.add_path(path)
-
-        vcs_context = {
-            "current_version": args.current_version,
-            "new_version": args.new_version,
-        }
-        vcs_context.update(time_context)
-        vcs_context.update(prefixed_environ())
-
-        commit_message = args.message.format(**vcs_context)
-
-        logger.info("{} to {} with message '{}'".format(
-            "Would commit" if args.dry_run else "Committing",
-            vcs.__name__,
-            commit_message,
+        logger.info("{} to config file {}:".format(
+            "Would write" if not write_to_config_file else "Writing",
+            known_args.config_file,
         ))
 
-        if not args.dry_run:
-            vcs.commit(message=commit_message)
+        config.write(new_config)
+        logger.info(new_config.getvalue())
 
-        if args.tag:
-            tag_name = args.tag_name.format(**vcs_context)
-            logger.info("{} '{}' in {}".format(
-                "Would tag" if args.dry_run else "Tagging",
-                tag_name,
-                vcs.__name__
-            ))
+        if write_to_config_file:
+            with io.open(known_args.config_file, 'wb') as f:
+                f.write(new_config.getvalue().encode('utf-8'))
 
-            if not args.dry_run:
-                vcs.tag(tag_name)
+    except UnicodeEncodeError:
+        warnings.warn(
+            "Unable to write UTF-8 to config file, because of an old configparser version. "
+            "Update with `pip install --upgrade configparser`."
+        )
+
+    if config_file_exists:
+        commit_files.append(known_args.config_file)
+
+    if not vcs:
+        return
+
+    assert vcs.is_usable(), "Did find '{}' unusable, unable to commit.".format(vcs.__name__)
+
+    do_commit = (not args.dry_run) and args.commit
+    do_tag = (not args.dry_run) and args.tag
+
+    logger.info("{} {} commit".format(
+        "Would prepare" if not do_commit else "Preparing",
+        vcs.__name__,
+    ))
+
+    for path in commit_files:
+        logger.info("{} changes in file '{}' to {}".format(
+            "Would add" if not do_commit else "Adding",
+            path,
+            vcs.__name__,
+        ))
+
+        if do_commit:
+            vcs.add_path(path)
+
+    vcs_context = {
+        "current_version": args.current_version,
+        "new_version": args.new_version,
+    }
+    vcs_context.update(time_context)
+    vcs_context.update(prefixed_environ())
+
+    commit_message = args.message.format(**vcs_context)
+
+    logger.info("{} to {} with message '{}'".format(
+        "Would commit" if not do_commit else "Committing",
+        vcs.__name__,
+        commit_message,
+    ))
+
+    if do_commit:
+        vcs.commit(message=commit_message)
+
+    tag_name = args.tag_name.format(**vcs_context)
+    logger.info("{} '{}' in {}".format(
+        "Would tag" if not do_tag else "Tagging",
+        tag_name,
+        vcs.__name__
+    ))
+
+    if do_tag:
+        vcs.tag(tag_name)
 
